/*
 * (c) Stephen Chenney, University of Wisconsin at Madison, 2001
 *
 */

#include "SphereWindow.h"
#include <Fl/math.h>
#include <Fl/gl.h>
#include <GL/glu.h>
#include <stdio.h>

const double SphereWindow::FOV_X = 45.0;

SphereWindow::SphereWindow(int x, int y, int width, int height, char *label)
	: Fl_Gl_Window(x, y, width, height, label)
{
    button = -1;

    // Initial viewing parameters.
    phi = 45.0f;
    theta = 0.0f;
    dist = 5.0f;

    smooth = false;
}


void
SphereWindow::draw(void)
{
    double  eye[3];
    float   color[4], dir[4];

    if ( ! valid() )
    {
	// Stuff in here doesn't change from frame to frame, and does not
	// rely on any coordinate system. It only has to be done if the
	// GL context is damaged.

	double	fov_y;

	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);

	// Turn on depth testing
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LESS);

	// Enable lighting with one light.
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHTING);

	// Ambient and diffuse lighting track the current color.
	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);

	// Turn off normal vector normalization.
	glDisable(GL_NORMALIZE);

	// Set up the viewport.
	glViewport(0, 0, w(), h());

	// Set up the persepctive transformation.
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	fov_y = 360.0f / M_PI * atan(h() * tan(FOV_X * M_PI / 360.0) / w());
	gluPerspective(fov_y, 1.0, 0.1, 1000.0);

    }

    // Stuff out here relies on a coordinate system or must be done on every
    // frame.

    // Clear the screen. Color and depth.
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

    // Set up the viewing transformation. The viewer is at a distance
    // dist from (x_at, y_ay, 2.0) in the direction (theta, phi) defined
    // by two angles. They are looking at (x_at, y_ay, 2.0) and z is up.
    eye[0] = dist * cos(theta * M_PI / 180.0) * cos(phi * M_PI / 180.0);
    eye[1] = dist * sin(theta * M_PI / 180.0) * cos(phi * M_PI / 180.0);
    eye[2] = dist * sin(phi * M_PI / 180.0);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    gluLookAt(eye[0], eye[1], eye[2], 0.0, 0.0, 0.0, 0.0, 0.0, 1.0);

    color[0] = 0.8f; color[1] = 0.8f; color[2] = 0.8f; color[3] = 1.0f;
    glLightfv(GL_LIGHT0, GL_DIFFUSE, color);
    color[0] = 0.0f; color[1] = 0.0f; color[2] = 0.0f; color[3] = 1.0f;
    glLightfv(GL_LIGHT0, GL_AMBIENT, color);
    color[0] = 0.0f; color[1] = 0.0f; color[2] = 0.0f; color[3] = 1.0f;
    glLightfv(GL_LIGHT0, GL_SPECULAR, color);
    dir[0] = 0.6f; dir[1] = 0.65f; dir[2] = 0.5f; dir[3] = 0.0f;
    glLightfv(GL_LIGHT0, GL_POSITION, dir);

    glColor3f(1.0f, 1.0f, 1.0f);
    sphere.Render(smooth);
}


void
SphereWindow::Drag()
{
    int	    dx = x_down - x_last;
    int     dy = y_down - y_last;

    switch ( button )
    {
      case FL_LEFT_MOUSE:
	// Left button changes the direction the viewer is looking from.
	theta = theta_down + 360.0f * dx / (float)w();
	while ( theta >= 360.0f )
	    theta -= 360.0f;
	while ( theta < 0.0f )
	    theta += 360.0f;
	phi = phi_down + 90.0f * dy / (float)h();
	if ( phi > 89.0f )
	    phi = 89.0f;
	if ( phi < 0.0f )
	    phi = 0.0f;
	break;
      case FL_RIGHT_MOUSE:
	dist = dist_down - ( 0.5f * dist_down * dy / (float)h() );
	if ( dist < 1.0f )
	    dist = 1.0f;
	break;
      default:;
    }
}


int
SphereWindow::handle(int event)
{
    // Event handling routine. Only looks at mouse events.
    // Stores a bunch of values when the mouse goes down and keeps track
    // of where the mouse is and what mouse button is down, if any.
    switch ( event )
    {
      case FL_PUSH:
        button = Fl::event_button();
	x_last = x_down = Fl::event_x();
	y_last = y_down = Fl::event_y();
	phi_down = phi;
	theta_down = theta;
	dist_down = dist;
	return 1;
      case FL_DRAG:
	x_last = Fl::event_x();
	y_last = Fl::event_y();
	Drag();
	redraw();
	return 1;
      case FL_RELEASE:
        button = -1;
	return 1;
      case FL_FOCUS:
	return 1;
      case FL_UNFOCUS:
	return 1;
      case FL_KEYBOARD:
      	if ( Fl::event_key() == 's' )
	{
	    sphere.Subdivide(1);
	    redraw();
	    return 1;
	}
      	if ( Fl::event_key() == 'q' )
	    exit(0);
	if ( Fl::event_key() == 'n' )
	{
	    smooth = ! smooth;
	    redraw();
	    return 1;
	}
	return 0;
    }

    // Pass any other event types on the superclass.
    return Fl_Gl_Window::handle(event);
}


